#ifndef GREETERHANDLER_H
#define GREETERHANDLER_H

#include <QSocketNotifier>

#include "managers/localauthmanager.h"
#include "structures.h"


class GreeterHandler : public QObject
{
    Q_OBJECT

public:
    explicit GreeterHandler(const int dmSocket, QObject* parent = nullptr);
    virtual ~GreeterHandler();

signals:
    void successLogin();
    void loginFailed();
    void passwordChangeRequestedByPam(const bool withAuth);
    void pamErrorReceived(const QString &message);

public:
    void start();
    void stop();
    static const QString readLastUserSession(const QString& username);
    static const DpyDescriptions getDisplaySessions(const int flags);
    static void doShutdown(const int type, const QString& os = QString());

public slots:
    void handleLoginRequest(const AuthData& data);
    void setSessionType(const QString& session);
    void handleConsoleLoginRequest();
    void switchDisplay(const int vt);
    void onNewPasswordInsertionCompleted(const QString &login,
                                         const QString &oldPassword,
                                         const QString &newPassword,
                                         const QString &confirmPassword);
    void onNewPasswordInsertionRejected();

protected slots:
    void handleSocket();

protected:
    QSocketNotifier m_socketNotifier;

private:
    int m_passwordChangeStage = -1;

    bool m_passwordChangeWithAuth = false;
    bool m_startedPasswordChange = false;
    bool m_lastGetTextPamEcho = false;

    QString m_login;
    QString m_oldPassword;
    QString m_newPassword;
    QString m_confirmPassword;
    QString m_lastGetTextPamMessage;

    void handleGetTextPamResponse(const char *response, bool echo, bool nonBlocking);
    void sendMessageToPam(const char *text, int tag);
    void sendDataToPam();
};

#endif // GREETERHANDLER_H
