#ifndef THEMEWIDGET_H
#define THEMEWIDGET_H

#include "widgets/basethemewidget.h"
#include "widgets/datetimewidgethandler.h"
#include "managers/usersmanager.h"
#include "managers/localauthmanager.h"

#include <QTimer>


class CommonWidget;
class LoginWidget;
class UserCellWidget;
class HorizontalUserWidget;
class VirtualKeyboardManager;
class KeyboardManager;
class ShutdownManager;
class SessionsManager;
class ConnectionManager;
class ConsoleAuthManager;
class DomainManager;


class ThemeWidget : public BaseThemeWidget
{
    Q_OBJECT
    Q_PLUGIN_METADATA(IID BaseThemeWidget_iid)
    Q_INTERFACES(BaseThemeWidget)

public:
    explicit ThemeWidget(QWidget* parent = nullptr);
    ~ThemeWidget() override;

public:
    void load(const ThemeDescription& themeDescription, GreeterManagers* managers, const EnvironmentData& environment) override;

protected slots:
    void performLoginRequest(const AuthData& data);
    void performLoginFailed();
    void performSuccessLogin();
    void loginFailedTimerTimeout();
    void virtualKeyboardErrorHandle(const QString& error);
    void capsLockStateChangedHandle(const bool lock);
    void rebootHandle();
    void shutdownHandle();
    void sleepHandle();
    void suspendToRamHandle();
    void consoleLoginhandle();
    void exitActionTriggered();
    void changeAuthPageActionTriggered();
    void authPageChanged();
    void openNewPasswordDialog(const bool withAuth);

protected:
    template<typename T>
    T findWidget(const QString& name) const;
    template<typename T>
    T findManager(const QString& id) const;

    void loadTranslations();
    void setBackgroundImage(const QString& path);

    void resizeEvent(QResizeEvent* event) override;

protected:
    QWidget* m_backgroundWidget;

    EnvironmentData m_environment;
    ThemeDescription m_themeDescription;
    DateTimeWidgetHandler m_dateTimeWidget;
    UsersManager* m_usersManager;
    LocalAuthManager* m_localAuthManager;
    VirtualKeyboardManager* m_virtualKeyboardManager;
    KeyboardManager* m_keyboardManager;
    ShutdownManager* m_shutdownManager;
    SessionsManager* m_sessionsManager;
    ConnectionManager* m_connectionManager;
    ConsoleAuthManager* m_consoleAuthManager;
    DomainManager *m_domainManager;
    CommonWidget* m_commonWidget;
    LoginWidget* m_loginWidget;
    QTimer m_loginFailedTimer;
    QAction* m_changeAuthPageAction;

signals:
    void newPasswordInsertionCompleted(const QString &login,
                                       const QString &oldPassword,
                                       const QString &newPassword,
                                       const QString &confirmPassword);

private slots:
    void onNewPasswordInsertionCompleted(const QString &oldPassword,
                                         const QString &newPassword,
                                         const QString &confirmPassword);
};

#endif // THEMEWIDGET_H
