#include "mainwindow.h"

#include "themewidgetcreator.h"
#include "exceptions.h"

#include "managers/greetermanagers.h"
#include "widgets/basethemewidget.h"
#include "managers/sessionsmanager.h"
#include "managers/usersmanager.h"
#include "managers/shutdownmanager.h"
#include "managers/virtualkeyboardmanager.h"
#include "managers/keyboardmanager.h"
#include "managers/connectionmanager.h"
#include "managers/consoleauthmanager.h"
#include "managers/localauthmanager.h"

#include <QMessageBox>
#include <QGridLayout>
#include <QLocale>


MainWindow::MainWindow(QWidget* parent) :
    QDialog(parent)
{
    setGeometry(100, 100, 1280, 720);
    setObjectName("mainDialog");
}

MainWindow::~MainWindow()
{
    delete m_managers;
}

void MainWindow::load()
{
    try {
        ThemeInfo themeInfo("/home/developer/share/work/fly-qdm/themes/fly-modern");
        themeInfo.read();

        const QString pluginPath("/home/developer/work/build-fly-qdm-Desktop-Debug/dm_greet_modern/debug/libfly-dmgreet_modern.so");

        UserInfo::Filter usersFilter;
        usersFilter.showUsers = true;
        usersFilter.lowUserId = 1000;
        usersFilter.highUserId = 2000;
        usersFilter.showRoot = false;
        usersFilter.sortUsers = true;
        auto users = UserInfo::getUserList(usersFilter);
        users += users += users += users;
        const auto icons = UserIcon::getUserIcons("/usr/share/fly-dm/faces");
        const auto usersManager = new UsersManager(users, icons, nullptr);

        SessionList sessions;
        for (int i = 0; i < 5; i++) {
            SessionDescription description;
            description.name[""] = QString::number(i);
            description.type = QString::number(i);
            sessions << description;
        }
        const auto sessionsManager = new SessionsManager(sessions);

        ConnectionManager::ConnectionData connectionData;
        connectionData.allowClose = true;
        const auto connectionManager = new ConnectionManager(connectionData, DpyDescriptions());

        m_managers = new GreeterManagers({new LocalAuthManager(),
                                          sessionsManager,
                                          usersManager,
                                          new ShutdownManager(),
                                          new VirtualKeyboardManager(""),
                                          new KeyboardManager(XKeyboard::instance()),
                                          connectionManager,
                                          new ConsoleAuthManager(true)});

        const auto locale = QLocale().name();
        EnvironmentData enviroment;
        enviroment.locale = locale.size() >= 2 ? locale.left(2) : QString();

        const auto mainThemeWidget = ThemeWidgetCreator::create(themeInfo, pluginPath, m_managers, enviroment);

        const auto layout = new QGridLayout();
        layout->setMargin(0);
        layout->addWidget(mainThemeWidget);
        setLayout(layout);
    } catch (const DetailsException& exception) {
        QMessageBox::critical(this,
                              tr("Error"),
                              exception.fullDetails());
    }
}
